package info.aspirecn.cloud.yysj.controller;

import info.aspirecn.cloud.yysj.entity.*;
import info.aspirecn.cloud.yysj.service.WaybillSercice;
import info.aspirecn.cloud.yysj.utils.Constant;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 查询运单列表、运单详情，接口
 *
 * @author lixinxu
 */
@Api(tags = "运单查询接口")
@ApiResponses({
        @ApiResponse(code = 200, message = "成功处理请求"),
        @ApiResponse(code = 400, message = "没有找到该用户信息"),
        @ApiResponse(code = 401, message = "没有权限访问该服务"),
        @ApiResponse(code = 403, message = "权限不足无法访问该服务"),
        @ApiResponse(code = 404, message = "未发现该微服务"),
        @ApiResponse(code = 500, message = "服务器内部错误")
})
@RestController
public class WaybillController {

    @Autowired
    private WaybillSercice waybillSercice;

    @ApiOperation(value = "运单列表", notes = "查询运单信息列表")
    @PostMapping("/querywaybill")
    private ResponseInfo queryWaybillList(HttpServletResponse response,
                                          @ApiParam(value="用户ID")@RequestHeader()  String userId,
                                          @RequestBody WaybillQueryParam waybillQueryParam){

        waybillQueryParam.setUserId(userId);
        waybillQueryParam.setBeginTime(waybillQueryParam.getBeginTime());
        waybillQueryParam.setEndTime(waybillQueryParam.getEndTime());
        waybillQueryParam.setPageSize(waybillQueryParam.getPageSize());
        waybillQueryParam.setPageNum(waybillQueryParam.getPageNum());

        //map里有数据列表和查询总条数
        Map<String, Object> reMap = waybillSercice.queryWaybillList(waybillQueryParam);
        if (reMap == null){
            return new ResponseInfo().setMessage("数据深度不能大于1000条");
        }

        List<Waybill> waybillList = (List<Waybill>)reMap.get(Constant.DATA_LIST);
        long total = (long)reMap.get(Constant.TOTAL);

        //页面信息, 放到响应头
        response.addHeader("pageSize", Integer.toString(waybillQueryParam.getPageSize()));
        response.addHeader("pageNum", Integer.toString(waybillQueryParam.getPageNum()));
        response.addHeader("pageTotal", Long.toString(total%waybillQueryParam.getPageSize() > 0 ? total/waybillQueryParam.getPageSize()+1 : total/waybillQueryParam.getPageSize()));
        response.addHeader("total", Long.toString(total));

        return new ResponseInfo().setData(waybillList);
    }

    @ApiOperation(value = "运单详情", notes = "查询运单详情")
    @GetMapping("/querywaybillinfo/{waybillId}")
    private ResponseInfo querywaybillinfo(@PathVariable(required = true) String waybillId){
        WaybillInfo waybillInfo = waybillSercice.querywaybillinfo(waybillId);
        return new ResponseInfo().setData(waybillInfo);
    }

}
